using System;
using System.Drawing;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;

namespace ReproSyncBug
{
	/// <summary>
	/// Summary description for PartForm.
	/// </summary>
	public class PartForm : System.Windows.Forms.Form
	{
		private Syncfusion.Windows.Forms.Tools.DockingManager dockingManager1;
		private System.Windows.Forms.Panel pnlMasterCycles;
		private System.Windows.Forms.Panel pnlInputs;
		private Syncfusion.Windows.Forms.Grid.GridDataBoundGrid dbgMasterCycles;
		private Syncfusion.Windows.Forms.Grid.GridDataBoundGrid dbgMasterReadings;
		private System.ComponentModel.IContainer components;

		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMCCycleID;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMCSubCycle;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMCConcurrency;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMCStation;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcCycleType;

		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMRInput;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMRCycleID;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gbcMRSubCycle;

		private const int iMasterCycleGridCycleIDColIndex = 1;
		private const int iMasterCycleGridSubCycleColIndex = 2;
		private const int iMasterCycleGridCycleTypeColIndex = 5;

		private string strMasterReadingsEmptyFilter;
		private string strMasterReadingShowallFilter;

		private ProgramData progData;
		private DataRowView drvPart;
		private DataView dvMasterCycles;
		private DataView dvMasterReadings;
		private DataView dvInputs;
		private string strPartID = "PartA";

		public PartForm(Form1 parent, ProgramData pData)
		{
			this.MdiParent = parent;
			this.progData = pData;
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			string filter = "PartID = 'PartA'";
			DataView dvPart = new DataView(progData.Part,filter,"",DataViewRowState.CurrentRows);
			dvPart.AllowEdit = true;
			this.drvPart = dvPart[0];
			dvMasterCycles = drvPart.CreateChildView("PartMasterCycle");
			dbgMasterCycles.DataSource = dvMasterCycles;
			//MASTER READINGS 
			dvMasterReadings = drvPart.CreateChildView("PartMasterReading");
			strMasterReadingsEmptyFilter = @"PartID = 'x9x9x9' and PartCycleID = 'x9x9x9'"; //CycleID = 'x9x9x9' and SubCycle = 'x9x9x9'";
			strMasterReadingShowallFilter = "PartID = '" + strPartID + "'";
			dbgMasterReadings.DataSource = dvMasterReadings;
			//set wrap behavior at first and last column
			this.dbgMasterReadings.Model.Options.WrapCellBehavior = GridWrapCellBehavior.WrapRow;
			
			this.InitControls();
		}


		private void InitControls()
		{

			//set up combo box for StationID
			dbgMasterCycles.GridBoundColumns["StationID"].StyleInfo.DataSource = this.progData.Station.DefaultView;
			dbgMasterCycles.GridBoundColumns["StationID"].StyleInfo.ValueMember = "StationID";
			dbgMasterCycles.GridBoundColumns["StationID"].StyleInfo.DisplayMember = "StationID";
			//set up Concurrency list
			StringCollection ConcurrencyTypes = new StringCollection();
			ConcurrencyTypes.Add("Sequenced");
			ConcurrencyTypes.Add("Concurrent");
			dbgMasterCycles.GridBoundColumns["Concurrency"].StyleInfo.ChoiceList = ConcurrencyTypes;
			//set wrap behavior at first and last column
			this.dbgMasterCycles.Model.Options.WrapCellBehavior = GridWrapCellBehavior.WrapRow;

			dvInputs = progData.Input.DefaultView;

			dbgMasterReadings.GridBoundColumns["InputID"].StyleInfo.DataSource = dvInputs;
			dbgMasterReadings.GridBoundColumns["InputID"].StyleInfo.ValueMember = "InputID";
			dbgMasterReadings.GridBoundColumns["InputID"].StyleInfo.DisplayMember = "InputID";
			
			StringCollection MasterCycleTypeChoices = new StringCollection();
			MasterCycleTypeChoices.Add("STATIC");
			MasterCycleTypeChoices.Add("SPATIAL");
			MasterCycleTypeChoices.Add("COUNT(.005,200)");
			//add CycleTypeChoices to Master Cycle grid
			this.dbgMasterCycles.GridBoundColumns["CycleType"].StyleInfo.ChoiceList = MasterCycleTypeChoices;

			//Hide columns auto-populated in events
			this.dbgMasterCycles.Model.Cols.Hidden["SubCycle"] = true;
			this.dbgMasterCycles.Model.Cols.Hidden["Concurrency"] = true;
			dbgMasterReadings.Model.Cols.Hidden["CycleID"] = true;
			dbgMasterReadings.Model.Cols.Hidden["SubCycle"] = true;

			RegisterEvents();
		}

		private void RegisterEvents()
		{
			//EditModeChanged used to set default values in the dbgMasterCycle grid
			this.dbgMasterCycles.Binder.EditModeChanged += new EventHandler(OnMasterCycles_EditModeChanged);
			//These three handlers update the MasterReadings grid and related IOMasterReadings grid when things change 
			//in the MasterCycle grid
			this.dbgMasterCycles.Binder.CurrentPositionChanged += new EventHandler(OnCurMasterCycleRowChange);
			this.dbgMasterCycles.RowSaved += new GridRowEventHandler(OnMasterCycleRowSaved);
			this.dbgMasterCycles.CurrentCellAcceptedChanges += new CancelEventHandler(OnMasterCycles_FieldValChangeAccepted);
	//		this.dbgMasterCycles.PrepareViewStyleInfo += new GridPrepareViewStyleInfoEventHandler(dbgMasterCycles_PrepareViewStyleInfo);
			//EditModeChanged used to set default values in the dbgMasterReadings grid
			this.dbgMasterReadings.Binder.EditModeChanged += new EventHandler(OnMasterReadings_EditModeChanged);

		}


		private void SetMasterReadingsColWidths()
		{
			this.dbgMasterCycles.Model.ColWidths.ResizeToFit(GridRangeInfo.Table(),GridResizeToFitOptions.IncludeHeaders);
			this.dbgMasterReadings.Model.ColWidths.ResizeToFit(GridRangeInfo.Table(),GridResizeToFitOptions.IncludeHeaders);
		}

		private void OnMasterCycles_EditModeChanged(object sender, EventArgs e)
		{
			Syncfusion.Windows.Forms.Grid.GridModelDataBinder binder = sender as Syncfusion.Windows.Forms.Grid.GridModelDataBinder;
			int pos = binder.CurrentPosition;
			//if this is the last record (no guarantee that its a new record)
			if (binder.IsEditing 
				&& binder.RecordCount-1 == pos) 
			{	//set the PartID
				//	DataRow dr = dvMasterCycles[pos].Row;
				ProgramData.MasterCycleRow mcr = dvMasterCycles[pos].Row as ProgramData.MasterCycleRow;
				if (mcr.RowState == DataRowState.Detached) //If a new row, set defaults
				{
					//dr["PartID"] = strPartID;
					mcr.PartID = this.strPartID;
					//dr["PartCycleID"] = progMan.DefaultPartCycleID;//.ToString();
					mcr.PartCycleID = "1";//progMan.DefaultPartCycleID;
					//Always default Concurrency to "Sequenced"
					mcr.Concurrency = "Sequenced";
					//If not using MultipleReadCycles within a MasterCycle, default the SubCycle value to "A"
				//	if (this.progMan.EditoroptionsRow.UseMasterSubCycles==false)
					
						//dr["SubCycle"] = "A";
					mcr.SubCycle = "A";
					//Testing to fix Syncfusion bug from 3.0.1.0
					//mcr.CycleType = "STATIC"; - DID NOT WORK!
				}
			}
		}

		private void OnCurMasterCycleRowChange(object sender, System.EventArgs e)
		{
			//MessageBox.Show("OnCurChange");
			this.updateMasterReadingsGridView();
		}

		private void OnMasterCycleRowSaved(object sender, GridRowEventArgs e)
		{
			//MessageBox.Show("OnMuxRowSaved");
			this.updateMasterReadingsGridView();
		}

		private void OnMasterCycles_FieldValChangeAccepted(object sender, CancelEventArgs e)
		{
			//If the Cycle field value was changed, update the Master Readings and IO-MasterReadings grid
			//MessageBox.Show(dbgTestReadings.CurrentCell.ColIndex.ToString());
			int CurCellColIndex = dbgMasterCycles.CurrentCell.ColIndex;
			if (CurCellColIndex == iMasterCycleGridCycleIDColIndex)
			{
				this.updateMasterReadingsGridView();
			}
//			if (CurCellColIndex == iMasterCycleGridSubCycleColIndex)
//				if (this.progMan.EditoroptionsRow.UseMasterSubCycles==true)
//					this.updateMasterReadingsGridView();
			//MessageBox.Show("CurrentCellAcceptedChanges");
			//e.Cancel = true;
		}

		private void OnMasterReadings_EditModeChanged(object sender, EventArgs e)
		{
			Syncfusion.Windows.Forms.Grid.GridModelDataBinder binder = sender as Syncfusion.Windows.Forms.Grid.GridModelDataBinder;
			int pos = binder.CurrentPosition;
			//if this is the last record (no guarantee that its a new record)
			if (binder.IsEditing 
				&& binder.RecordCount-1 == pos) 
			{	//set the PartID
				DataRow dr = dvMasterReadings[pos].Row;
				if (dr.RowState == DataRowState.Detached) //If a new row, set defaults
				{
					dr["PartID"] = strPartID;
					//Comment this out when Part Cycles are implemented (ie. when if statement added)
					dr["PartCycleID"] = "1";
					CurrencyManager cm = (CurrencyManager)this.BindingContext[this.dbgMasterCycles.DataSource];
					//if(cm.Current.GetType() != typeof(DataRowView)) return;
					DataRowView drvMasterCycles = (DataRowView)cm.Current;
					//if showall is not checked, default the cycle from the current row in the cycle grid
//					if (false == cboxMasterReadingsShowAll.Checked)
//					{ //must default the Cycle value because this columns are disabled when the narrow view is applied
						
						dr["PartCycleID"] = drvMasterCycles["PartCycleID"];
						dr["CycleID"] = drvMasterCycles["CycleID"];
						dr["SubCycle"] = drvMasterCycles["SubCycle"];
			//		}
						//If showall is checked, but not using MultipleReadCycles, then must set the hidden column field
//					else if (this.progMan.EditoroptionsRow.UseMasterSubCycles==false)
//						dr["SubCycle"] = drvMasterCycles["SubCycle"];
				}
			}
		}

		private void updateMasterReadingsGridView()
		{
//			if (cboxMasterReadingsShowAll.Checked)
//				dvMasterReadings.RowFilter = strMasterReadingShowallFilter; 
//			else
				updateMasterReadingGridNarrowView();
		}

		/// <summary>
		/// Filters the MasterReading grid to show only those readings related to the cycle having the focus.
		/// If there are not cycles, the grid is cleared
		/// </summary>
		private void updateMasterReadingGridNarrowView()
		{
			//CurrentPosition can apparently come back 0 when the record count is 0 so prevent that problem
			if (dbgMasterCycles.Binder.RecordCount < 1)
			{  //Narrow view when no Readings exist is empty
				dvMasterReadings.RowFilter = strMasterReadingsEmptyFilter;
				return;
			}
			int pos = dbgMasterCycles.Binder.CurrentPosition;
			if (dvMasterCycles.Count > pos)
			{
				DataRow dr = dvMasterCycles[pos].Row;
				string filter = "PartID = '" + strPartID + "' and PartCycleID = '" + dr["PartCycleID"] + "' and CycleID = '" + dr["CycleID"] + "' and SubCycle = '" + dr["SubCycle"] + "'";
				dvMasterReadings.RowFilter = filter;
			}
			else
			{
				dvMasterReadings.RowFilter = strMasterReadingsEmptyFilter;
			}
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(PartForm));
			this.dockingManager1 = new Syncfusion.Windows.Forms.Tools.DockingManager(this.components);
			this.pnlInputs = new System.Windows.Forms.Panel();
			this.dbgMasterReadings = new Syncfusion.Windows.Forms.Grid.GridDataBoundGrid();
			this.gbcMRCycleID = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gbcMRSubCycle = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gbcMRInput = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.pnlMasterCycles = new System.Windows.Forms.Panel();
			this.dbgMasterCycles = new Syncfusion.Windows.Forms.Grid.GridDataBoundGrid();
			this.gbcMCCycleID = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gbcMCSubCycle = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gbcMCConcurrency = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gbcMCStation = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gbcCycleType = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			((System.ComponentModel.ISupportInitialize)(this.dockingManager1)).BeginInit();
			this.pnlInputs.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.dbgMasterReadings)).BeginInit();
			this.pnlMasterCycles.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.dbgMasterCycles)).BeginInit();
			// 
			// dockingManager1
			// 
			this.dockingManager1.DockLayoutStream = ((System.IO.MemoryStream)(resources.GetObject("dockingManager1.DockLayoutStream")));
			this.dockingManager1.DockToFill = true;
			this.dockingManager1.HostControl = this;
			this.dockingManager1.ShowToolTips = true;
			this.dockingManager1.SetDockLabel(this.pnlMasterCycles, "pnlMasterCycles");
			this.dockingManager1.SetDockLabel(this.pnlInputs, "pnlInputs");
			// 
			// pnlInputs
			// 
			this.pnlInputs.Controls.Add(this.dbgMasterReadings);
			this.dockingManager1.SetEnableDocking(this.pnlInputs, true);
			this.pnlInputs.Location = new System.Drawing.Point(1, 24);
			this.pnlInputs.Name = "pnlInputs";
			this.pnlInputs.Size = new System.Drawing.Size(427, 386);
			this.pnlInputs.TabIndex = 1;
			// 
			// dbgMasterReadings
			// 
			this.dbgMasterReadings.AllowDragSelectedCols = true;
			this.dbgMasterReadings.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.dbgMasterReadings.GridBoundColumns.AddRange(new Syncfusion.Windows.Forms.Grid.GridBoundColumn[] {
																													 this.gbcMRCycleID,
																													 this.gbcMRSubCycle,
																													 this.gbcMRInput});
			this.dbgMasterReadings.Location = new System.Drawing.Point(8, 8);
			this.dbgMasterReadings.Name = "dbgMasterReadings";
			this.dbgMasterReadings.OptimizeInsertRemoveCells = true;
			this.dbgMasterReadings.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			this.dbgMasterReadings.Size = new System.Drawing.Size(397, 352);
			this.dbgMasterReadings.SmartSizeBox = false;
			this.dbgMasterReadings.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.DoubleClick;
			this.dbgMasterReadings.TabIndex = 0;
			this.dbgMasterReadings.Text = "gridDataBoundGrid1";
			this.dbgMasterReadings.UseListChangedEvent = true;
			// 
			// gbcMRCycleID
			// 
			this.gbcMRCycleID.HeaderText = "CycleID";
			this.gbcMRCycleID.MappingName = "CycleID";
			this.gbcMRCycleID.StyleInfo.CellType = "NumericUpDown";
			// 
			// gbcMRSubCycle
			// 
			this.gbcMRSubCycle.HeaderText = "SubCycle";
			this.gbcMRSubCycle.MappingName = "SubCycle";
			// 
			// gbcMRInput
			// 
			this.gbcMRInput.HeaderText = "Input";
			this.gbcMRInput.MappingName = "InputID";
			this.gbcMRInput.StyleInfo.AutoSize = true;
			this.gbcMRInput.StyleInfo.CellType = "ComboBox";
			this.gbcMRInput.StyleInfo.DropDownStyle = Syncfusion.Windows.Forms.Grid.GridDropDownStyle.Exclusive;
			// 
			// pnlMasterCycles
			// 
			this.pnlMasterCycles.Controls.Add(this.dbgMasterCycles);
			this.dockingManager1.SetEnableDocking(this.pnlMasterCycles, true);
			this.pnlMasterCycles.Location = new System.Drawing.Point(1, 24);
			this.pnlMasterCycles.Name = "pnlMasterCycles";
			this.pnlMasterCycles.Size = new System.Drawing.Size(629, 386);
			this.pnlMasterCycles.TabIndex = 0;
			// 
			// dbgMasterCycles
			// 
			this.dbgMasterCycles.AllowDragSelectedCols = true;
			this.dbgMasterCycles.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.dbgMasterCycles.GridBoundColumns.AddRange(new Syncfusion.Windows.Forms.Grid.GridBoundColumn[] {
																												   this.gbcMCCycleID,
																												   this.gbcMCSubCycle,
																												   this.gbcMCConcurrency,
																												   this.gbcMCStation,
																												   this.gbcCycleType});
			this.dbgMasterCycles.Location = new System.Drawing.Point(8, 8);
			this.dbgMasterCycles.Name = "dbgMasterCycles";
			this.dbgMasterCycles.OptimizeInsertRemoveCells = true;
			this.dbgMasterCycles.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			this.dbgMasterCycles.Size = new System.Drawing.Size(611, 352);
			this.dbgMasterCycles.SmartSizeBox = false;
			this.dbgMasterCycles.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.DoubleClick;
			this.dbgMasterCycles.TabIndex = 0;
			this.dbgMasterCycles.Text = "gridDataBoundGrid1";
			this.dbgMasterCycles.UseListChangedEvent = true;
			// 
			// gbcMCCycleID
			// 
			this.gbcMCCycleID.HeaderText = "CycleID";
			this.gbcMCCycleID.MappingName = "CycleID";
			this.gbcMCCycleID.StyleInfo.CellType = "NumericUpDown";
			this.gbcMCCycleID.StyleInfo.CellValueType = null;
			this.gbcMCCycleID.StyleInfo.NumericUpDown.Minimum = 1;
			// 
			// gbcMCSubCycle
			// 
			this.gbcMCSubCycle.HeaderText = "SubCycle";
			this.gbcMCSubCycle.MappingName = "SubCycle";
			// 
			// gbcMCConcurrency
			// 
			this.gbcMCConcurrency.HeaderText = "Concurrency";
			this.gbcMCConcurrency.MappingName = "Concurrency";
			this.gbcMCConcurrency.StyleInfo.CellType = "ComboBox";
			this.gbcMCConcurrency.StyleInfo.DropDownStyle = Syncfusion.Windows.Forms.Grid.GridDropDownStyle.Exclusive;
			// 
			// gbcMCStation
			// 
			this.gbcMCStation.HeaderText = "Station";
			this.gbcMCStation.MappingName = "StationID";
			this.gbcMCStation.StyleInfo.CellType = "ComboBox";
			this.gbcMCStation.StyleInfo.DropDownStyle = Syncfusion.Windows.Forms.Grid.GridDropDownStyle.Exclusive;
			this.gbcMCStation.StyleInfo.NumericUpDown.Minimum = 1;
			// 
			// gbcCycleType
			// 
			this.gbcCycleType.HeaderText = "Cycle Type";
			this.gbcCycleType.MappingName = "CycleType";
			this.gbcCycleType.StyleInfo.CellType = "ComboBox";
			this.gbcCycleType.StyleInfo.CellValueType = typeof(string);
			this.gbcCycleType.StyleInfo.DropDownStyle = Syncfusion.Windows.Forms.Grid.GridDropDownStyle.Editable;
			this.gbcCycleType.StyleInfo.StrictValueType = false;
			// 
			// PartForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(6, 15);
			this.ClientSize = new System.Drawing.Size(1064, 411);
			this.Name = "PartForm";
			this.Text = "PartForm";
			this.Load += new System.EventHandler(this.PartForm_Load);
			((System.ComponentModel.ISupportInitialize)(this.dockingManager1)).EndInit();
			this.pnlInputs.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.dbgMasterReadings)).EndInit();
			this.pnlMasterCycles.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.dbgMasterCycles)).EndInit();

		}
		#endregion

		private void PartForm_Load(object sender, System.EventArgs e)
		{
			SetMasterReadingsColWidths();
		}
	}
}
